@@ -1,4 +1,4 @@
-/* 
+/*
  * Copyright (C) 2006, 2007, 2008 OpenedHand Ltd.
  * Copyright (C) 2009 Nokia Corporation.
  * Copyright (C) 2013 Intel Corporation.
@@ -42,6 +42,7 @@
 
 #include "gssdp-client.h"
 #include "gssdp-client-private.h"
+#include "gssdp-enums.h"
 #include "gssdp-error.h"
 #include "gssdp-socket-source.h"
 #include "gssdp-protocol.h"
@@ -74,12 +75,17 @@
 /* interface index for loopback device */
 #define LOOPBACK_IFINDEX 1
 
+static GInetAddress *SSDP_V6_LL_ADDR = NULL;
+static GInetAddress *SSDP_V6_SL_ADDR = NULL;
+static GInetAddress *SSDP_V4_ADDR = NULL;
+
 static void
 gssdp_client_initable_iface_init (gpointer g_iface,
                                   gpointer iface_data);
 
 struct _GSSDPClientPrivate {
         char              *server_id;
+        GSSDPUDAVersion    uda_version;
 
         GHashTable        *user_agent_cache;
         guint              socket_ttl;
@@ -117,9 +123,12 @@ enum {
         PROP_IFACE,
         PROP_NETWORK,
         PROP_HOST_IP,
+        PROP_HOST_MASK,
         PROP_ACTIVE,
         PROP_SOCKET_TTL,
         PROP_MSEARCH_PORT,
+        PROP_ADDRESS_FAMILY,
+        PROP_UDA_VERSION,
 };
 
 enum {
@@ -308,6 +317,12 @@ gssdp_client_get_property (GObject    *object,
         case PROP_MSEARCH_PORT:
                 g_value_set_uint (value, priv->msearch_port);
                 break;
+        case PROP_ADDRESS_FAMILY:
+                g_value_set_enum (value, priv->device.address_family);
+                break;
+        case PROP_UDA_VERSION:
+                g_value_set_enum (value, priv->uda_version);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
                 break;
@@ -343,6 +358,9 @@ gssdp_client_set_property (GObject      *object,
                         }
                         break;
                 }
+        case PROP_HOST_MASK:
+                priv->device.host_mask = g_value_dup_object (value);
+                break;
         case PROP_ACTIVE:
                 priv->active = g_value_get_boolean (value);
                 break;
@@ -352,6 +370,12 @@ gssdp_client_set_property (GObject      *object,
         case PROP_MSEARCH_PORT:
                 priv->msearch_port = g_value_get_uint (value);
                 break;
+        case PROP_ADDRESS_FAMILY:
+                priv->device.address_family = g_value_get_enum (value);
+                break;
+        case PROP_UDA_VERSION:
+                priv->uda_version = g_value_get_enum (value);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
                 break;
@@ -484,6 +508,23 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                                       G_PARAM_STATIC_NICK |
                                       G_PARAM_STATIC_BLURB));
 
+        /**
+         * GSSDPClient:host-mask:
+         *
+         * The network mask of the assoicated network interface.
+         **/
+        g_object_class_install_property
+                (object_class,
+                 PROP_HOST_MASK,
+                 g_param_spec_object ("host-mask",
+                                      "Host network mask",
+                                      "The IP netmask of the associated"
+                                      "network interface",
+                                      G_TYPE_INET_ADDRESS_MASK,
+                                      G_PARAM_WRITABLE |
+                                      G_PARAM_CONSTRUCT |
+                                      G_PARAM_STATIC_STRINGS));
+
         /**
          * GSSDPClient:active:
          *
@@ -545,6 +586,52 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                          G_PARAM_CONSTRUCT_ONLY |
                          G_PARAM_STATIC_STRINGS));
 
+        /**
+         * GSSDPClient:address-family:
+         *
+         * The IP protocol address family this client works on. When specified
+         * during construction without giving a concrete address, it will be
+         * used to determine the proper address.
+         *
+         * If not specified, will contain the currrent address family after
+         * the call to g_initable_init()<!-- -->. Use #G_SOCKET_FAMILY_INVALID
+         * to specifiy using the default socket family (legacy IP)
+         *
+         * Since: 1.1.1
+         */
+        g_object_class_install_property
+                (object_class,
+                 PROP_ADDRESS_FAMILY,
+                 g_param_spec_enum
+                        ("address-family",
+                         "IP Address family",
+                         "IP address family to prefer when creating the client",
+                         G_TYPE_SOCKET_FAMILY,
+                         G_SOCKET_FAMILY_INVALID,
+                         G_PARAM_READWRITE |
+                         G_PARAM_CONSTRUCT_ONLY |
+                         G_PARAM_STATIC_STRINGS));
+
+        /**
+         * GSSDPClient:uda-version:
+         *
+         * The UPnP version the client adheres to.
+         *
+         * Since: 1.1.1
+         */
+        g_object_class_install_property
+                (object_class,
+                 PROP_UDA_VERSION,
+                 g_param_spec_enum
+                        ("uda-version",
+                         "UDA version",
+                         "UPnP Device Architecture version on this client",
+                         GSSDP_TYPE_UDA_VERSION,
+                         GSSDP_UDA_VERSION_1_0,
+                         G_PARAM_READWRITE |
+                         G_PARAM_CONSTRUCT_ONLY |
+                         G_PARAM_STATIC_STRINGS));
+
         /**
          * GSSDPClient::message-received: (skip)
          * @client: The #GSSDPClient that received the message.
@@ -1058,6 +1145,39 @@ _gssdp_client_get_mcast_group (GSSDPClient *client)
         }
 }
 
+static GInetAddress *
+_gssdp_client_get_mcast_group_addr (GSSDPClient *client)
+{
+        GSocketFamily family;
+        GSSDPClientPrivate *priv = gssdp_client_get_instance_private (client);
+
+        family = g_inet_address_get_family (priv->device.host_addr);
+        if (family == G_SOCKET_FAMILY_IPV4) {
+                if (SSDP_V4_ADDR == NULL) {
+                        SSDP_V4_ADDR = g_inet_address_new_from_string (SSDP_ADDR);
+                }
+
+                return SSDP_V4_ADDR;
+        } else {
+                /* IPv6 */
+                /* According to Annex.A, we need to check the scope of the
+                 * address to use the proper multicast group */
+                if (g_inet_address_get_is_link_local (priv->device.host_addr)) {
+                            if (SSDP_V6_LL_ADDR == NULL) {
+                                    SSDP_V6_LL_ADDR = g_inet_address_new_from_string (SSDP_V6_LL);
+                            }
+
+                            return SSDP_V6_LL_ADDR;
+                } else {
+                            if (SSDP_V6_SL_ADDR == NULL) {
+                                    SSDP_V6_SL_ADDR = g_inet_address_new_from_string (SSDP_V6_SL);
+                            }
+
+                            return SSDP_V6_SL_ADDR;
+                }
+        }
+}
+
 /*
  * Generates the default server ID
  */
@@ -1080,7 +1200,7 @@ make_server_id (void)
         struct utsname sysinfo;
 
         uname (&sysinfo);
-        
+
         return g_strdup_printf ("%s/%s GSSDP/%s",
                                 sysinfo.sysname,
                                 sysinfo.version,
@@ -1211,19 +1331,22 @@ socket_source_cb (GSSDPSocketSource *socket_source, GSSDPClient *client)
                 goto out;
         }
 
-#if defined(HAVE_PKTINFO) && !defined(__APPLE__)
+#if defined(HAVE_PKTINFO)
         {
                 int i;
                 for (i = 0; i < num_messages; i++) {
                         gint msg_ifindex;
                         GInetAddress *local_addr;
+                        GInetAddress *group_addr;
+
+                        group_addr = _gssdp_client_get_mcast_group_addr (client);
 
                         if (GSSDP_IS_PKTINFO_MESSAGE (messages[i])) {
                                 GSSDPPktinfoMessage *msg;
 
                                 msg = GSSDP_PKTINFO_MESSAGE (messages[i]);
                                 msg_ifindex = gssdp_pktinfo_message_get_ifindex (msg);
-                                local_addr = gssdp_pktinfo_message_get_local_addr (msg);
+                                local_addr = gssdp_pktinfo_message_get_pkt_addr (msg);
                         } else if (GSSDP_IS_PKTINFO6_MESSAGE (messages[i])) {
                                 GSSDPPktinfo6Message *msg;
 
@@ -1239,10 +1362,12 @@ socket_source_cb (GSSDPSocketSource *socket_source, GSSDPClient *client)
                         if (!((msg_ifindex == priv->device.index ||
                                msg_ifindex == LOOPBACK_IFINDEX) &&
                               (g_inet_address_equal (local_addr,
-                                                     priv->device.host_addr))))
+                                                     priv->device.host_addr) ||
+                               g_inet_address_equal (local_addr, group_addr)))) {
                                 goto out;
-                        else
+                        } else {
                                 break;
+                        }
                 }
         }
 #else
@@ -1472,7 +1597,8 @@ init_network_info (GSSDPClient *client, GError **error)
          * interface.
          */
         if (priv->device.iface_name == NULL ||
-            priv->device.host_addr == NULL)
+            priv->device.host_addr == NULL  ||
+            priv->device.host_mask == NULL)
                 gssdp_net_get_host_ip (&(priv->device));
         else {
                 /* Ugly. Ideally, get_host_ip needs to be run everytime, but
@@ -1484,6 +1610,9 @@ init_network_info (GSSDPClient *client, GError **error)
                  */
                 priv->device.index =
                         gssdp_net_query_ifindex (&priv->device);
+
+                priv->device.address_family =  g_inet_address_get_family
+                                        (priv->device.host_addr);
         }
 
         if (priv->device.iface_name == NULL) {
